/* The Convex IRC System is (C) 2003 by HardCore SoftWare

   Your right to use, copy, and modify this software are protected under
   the GNU General Public License, available at www.gnu.org
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <netdb.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <signal.h>

#include "convex.h"



int openconnection(char *toconn, int port) {

  struct hostent *he;
  struct sockaddr_in their_addr; /* connector's address information */
  int sd,tp;

  if ((he=gethostbyname(toconn)) == NULL) {  /* get the host info */
    fprintf(stderr,"Unable to lookup host '%s'!\n", toconn);
    return 0;
  }

  if ((sd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
    fprintf(stderr,"Unable to create connection socket!\n");
    return 0;
  }

  their_addr.sin_family = AF_INET;         /* host byte order */
  their_addr.sin_port = htons(port);     /* short, network byte order */
  their_addr.sin_addr = *((struct in_addr *)he->h_addr);
  memset(&(their_addr.sin_zero), '\0', 8);

  if (connect(sd, (struct sockaddr *)&their_addr, sizeof(struct sockaddr)) == -1)
    return 0;

  if (sd == 0) {
    tp = dup(sd);
    close(sd);
    return tp;
  }

  die_setsock(sd);
  return sd;

}


FILE *buffer_sock(int sock) {
  FILE *fsock;

  fsock = fdopen(sock, "r");
  if (fsock == NULL) {
    fprintf(stderr, "Unable to buffer socket!\n");
    exit(0);
  }

  return fsock;

}


void proc_irc(struct map_struct *map, int sock, FILE *fsock,
              char *initial, char *server, int port) {

  char ibuf[2048], obuf[2048];
  int tp;

  snprintf(obuf, sizeof(obuf)-1, "%s\n", initial);
  write(sock, obuf, strlen(obuf));

  while (1) {

    if (fgets(ibuf, sizeof(ibuf)-1, fsock) == NULL) {
      close(sock);

      while ((sock = openconnection(server, port)) == 0) sleep(5);

      snprintf(obuf, sizeof(obuf)-1, "%s\n", initial);
      write(sock, obuf, strlen(obuf));

      fsock = buffer_sock(sock);

      continue;
    }

    for(tp=0; ibuf[tp] != '\n' && ibuf[tp] != '\r' && ibuf[tp] != '\0'; tp++)  ;
    ibuf[tp] = '\0';

    if (getmatch(ibuf, obuf, sizeof(obuf)-1, map)) {
      if (obuf[0] == '/') {
        /* MAKE THIS MORE EXTENSIBLE */
        if (strcmp(obuf+1, "rehash") == 0) map = rehash(map);
        else if (strncmp(obuf+1, "system", 6) == 0) system(obuf+8);
      } else {
        tp = strlen(obuf);
        if (obuf[tp-1] != '\n') {
          obuf[tp] = '\n';
          obuf[tp+1] = '\0';
          tp++;
        }
        write(sock, obuf, tp);
      }
    }

  }

}



void do_irc(struct map_struct *map) {

  char *server, *initial, *tp, *daemon, *newdir;
  int port, newuid=0, sock;
  FILE *fsock;

  server = get_const(map, "SERVER");
  if (server == NULL) {
    fprintf(stderr, "Unable to find a SERVER C: line in conf file!\n");
    exit(0);
  }
  initial = get_const(map, "INITIAL");
  if (initial == NULL) {
    fprintf(stderr, "Unable to find an INITIAL C: line in conf file!\n");
    exit(0);
  }
  tp = get_const(map, "PORT");
  if (tp == NULL) {
    fprintf(stderr, "Unable to find a PORT C: line in conf file!\n");
    exit(0);
  }
  port = atoi(tp);

  tp = get_const(map, "NEWUID");
  if (tp != NULL) newuid = atoi(tp);

  daemon = get_const(map, "DAEMON");
  newdir = get_const(map, "NEWDIR");

  if (newuid) {
    if ((getuid() == 0) || (geteuid() == 0)) setuid(newuid);
  }
      
  if (daemon != NULL) {
    if (fork()) exit(0);
    setsid();
  }
  if (newdir != NULL) chdir(newdir);

  if ((sock = openconnection(server, port)) == 0) {
    fprintf(stderr, "Unable to connect to %s:%d!\n", server, port);
    exit(0);
  }

  die_setqmsg(get_const(map, "QMSG"));

  if (get_const(map, "SNEAKY") != NULL) {
    signal(SIGINT, die_sneaky);
    signal(SIGQUIT, die_sneaky);
    signal(SIGTERM, die_sneaky);
  } else {
    signal(SIGINT, die_normal);
    signal(SIGQUIT, die_normal);
    signal(SIGTERM, die_normal);
  }

  fsock = buffer_sock(sock);

  proc_irc(map, sock, fsock, initial, server, port);

}
