/* The Convex IRC System is (C) 2003 by HardCore SoftWare

   Your right to use, copy, and modify this software are protected under
   the GNU General Public License, available at www.gnu.org
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include "convex.h"



void destroy_binds(struct bind_struct *binds) {
  struct bind_struct *tp, *orig;

  orig = binds;

  binds = binds->next;

  while (binds!=NULL) {
    tp = binds->next;
    free(binds);
    binds = tp;
  }

  orig->next = NULL;

}



void add_binding(struct bind_struct *binds, char *name, int namelen,
                                            char *text, int textlen) {

  while(binds->next != NULL) binds = binds->next;
  binds->next = (struct bind_struct *) malloc(sizeof(struct bind_struct));
  binds = binds->next;

  binds->name = name;
  binds->namelen = namelen;
  binds->text = text;
  binds->textlen = textlen;
  binds->next = NULL;

}



void do_expand(struct bind_struct *binds, char *res, char *out, int maxout) {

  int tp;
  struct bind_struct *bindtp;

  maxout-=3;

  while (maxout>0 && *res) {
    if (*res == '$') {
      for(tp=1; res[tp] != '\0' && res[tp] != '$'; tp++)  ;
      bindtp = binds;
      while (bindtp != NULL) {
        if (bindtp->name != NULL &&
            (strncmp(bindtp->name, res, bindtp->namelen) == 0)) {
          tp = 0;
          while(maxout>0 && tp<bindtp->textlen) {
            *out++ = bindtp->text[tp++];
            maxout--;
          }
          res+=bindtp->namelen;
          break;
        }
        bindtp = bindtp->next;
      }
    }
    *out++ = *res++;
    maxout--;
  }
  *out = '\0';

}




int wildtest(char *wild, char *test, struct bind_struct *binds) {

  int i, wl, tl;

  while(*wild != '\0'  ||  *test != '\0') {
    if (*wild == '$') {
      for(wl=1; wild[wl]!='$'; wl++)  ;
      wl++;
      if (wild[wl] == '$') return 0;   // Can't have "$bind1$$bind2$"
      for(tl=0; test[tl] != '\0' && test[tl]!=wild[wl]; tl++)  ;
      if (test[tl] != wild[wl]) return 0;

      if (wild[1] == '_' && wild[2] == '$') { }
      else add_binding(binds, wild, wl, test, tl);

      wild+=wl;
      test+=tl;
      continue;

    }

    if (*wild == '*') {

      /* --- Deal with multiple asterisks. --- */
      while (wild[1] == '*') wild++;

      /* --- Deal with terminating asterisks. --- */
      if (wild[1] == '\0') return 1;

      for(i=0; test[i]!='\0'; i++)
        if ((wild[1] == test[i]  ||  wild[1] == '?')
             &&  wildtest(wild+1, test+i, binds) == 1) return 1;

      return 0;
    }
    if (*wild != '?' && *wild != *test) return 0;
    wild++; test++;
  }

  if (*wild == *test) return 1;
  return 0;

}




int getmatch(char *in, char *out, int maxout, struct map_struct *map) {

  struct bind_struct binds;

  binds.next = NULL;
  binds.name = NULL;
  binds.namelen = 0;
  binds.text = NULL;
  binds.textlen = 0;

  while (map!=NULL) {
    if (map->pattern == NULL) {
      map = map->next;
      continue;
    }
    if (map->type == 0  &&  wildtest(map->pattern, in, &binds)) {
      do_expand(&binds, map->result, out, maxout);
      destroy_binds(&binds);
      if (out[0] == '\0') return 0;   /* Short circuit mapping */
      return 1;
    }
    destroy_binds(&binds);
    map = map->next;
  }
  return 0;

}
